import classNames from "classnames"
import React from "react"
import { ReactNode } from "react"

function Button({
  type,
  disabled,
  onClick,
  children,
  latching,
  isLatched,
}: {
  type: "red" | "orange" | "green" | "gray"
  disabled?: boolean
  onClick: () => void
  children: ReactNode
  latching?: boolean
  isLatched?: boolean
}) {
  return (
    <button
      disabled={disabled}
      className={classNames(
        "font-farro rounded-xl px-8 py-4 text-5xl font-medium duration-100",
        disabled
          ? "border-4 border-dashed"
          : latching
          ? isLatched
            ? "mx-1 my-0.5 border-t-4"
            : "mx-1 my-0.5 border-b-4"
          : "mx-1 my-0.5 border-b-4 active:border-b-0 active:border-t-4",
        {
          "border-red-600 bg-red-500": type === "red",
          "border-orange-400 bg-warn": type === "orange",
          "border-green-600 bg-green-500": type === "green",
          "border-gray-600 bg-gray-500": type === "gray",
        }
      )}
      onClick={onClick}
    >
      {children}
    </button>
  )
}

export default Button
