import { GameSettings } from "@components/Lobby/SettingsFrame/Setting"
import { GamePropsSchema, PlayerSchema } from "@lib/zodSchemas"
import type { Server as HTTPServer } from "http"
import type { Socket as NetSocket } from "net"
import type { NextApiResponse } from "next"
import { Session } from "next-auth"
import type {
  Server as IOServer,
  Server,
  Socket as SocketforServer,
} from "socket.io"
import type { Socket as SocketforClient } from "socket.io-client"

interface SocketServer extends HTTPServer {
  io?: IOServer
}

interface SocketWithIO extends NetSocket {
  server: SocketServer
}

export interface NextApiResponseWithSocket extends NextApiResponse {
  socket: SocketWithIO
}

export interface ServerToClientEvents {
  // noArg: () => void
  // basicEmit: (a: number, b: string, c: Buffer) => void
  // withAck: (d: string, ) => void
  gameSetting: (payload: GameSettings, hash: string) => void
  playerEvent: (
    event:
      | {
          type: "connect" | "leave"
          i: number
          payload: { users: PlayerSchema[] }
          hash: string
        }
      | {
          type: "disconnect"
          i: number
        }
  ) => void
  isReady: (payload: { i: number; isReady: boolean }) => void
  isConnected: (payload: { i: number; isConnected: boolean }) => void
}

export interface ClientToServerEvents {
  update: (callback: (game: GamePropsSchema) => void) => void
  isReady: (isReady: boolean) => void
  isConnected: (isReady: boolean) => void
  ping: (count: number, callback: (count: number) => void) => void
  join: (withAck: (ack: boolean) => void) => void
  gameSetting: (payload: GameSettings, callback: (hash: string) => void) => void
  leave: (withAck: (ack: boolean) => void) => void
}

interface InterServerEvents {
  // ping: () => void
}

interface SocketData {
  props: {
    userId: string
    gameId: string
    index: number
  }
  user: Session["user"]
  gameId: string | null
  index: number
}

export type sServer = Server<
  ClientToServerEvents,
  ServerToClientEvents,
  InterServerEvents,
  SocketData
>
export type sSocket = SocketforServer<
  ClientToServerEvents,
  ServerToClientEvents,
  InterServerEvents,
  SocketData
>

export type cSocket = SocketforClient<
  ServerToClientEvents,
  ClientToServerEvents
>
