import type {
  Hit,
  HitDispatch,
  Mode,
  Position,
  Target,
  TargetList,
} from "../../interfaces/frontend"
import { count } from "@components/Gamefield/Gamefield"
import { PointerProps } from "@components/Gamefield/GamefieldPointer"

export function borderCN(count: number, x: number, y: number) {
  if (x === 0) return "left"
  if (y === 0) return "top"
  if (x === count + 1) return "right"
  if (y === count + 1) return "bottom"
  return ""
}
export function cornerCN(count: number, x: number, y: number) {
  if (x === 0 && y === 0) return "left-top-corner"
  if (x === count + 1 && y === 0) return "right-top-corner"
  if (x === 0 && y === count + 1) return "left-bottom-corner"
  if (x === count + 1 && y === count + 1) return "right-bottom-corner"
  return ""
}
export function fieldIndex(count: number, x: number, y: number) {
  return y * (count + 2) + x
}
export function hitReducer(formObject: Hit[], action: HitDispatch) {
  switch (action.type) {
    case "fireMissile":
    case "htorpedo":
    case "vtorpedo": {
      const result = [...formObject, ...action.payload]
      return result
    }

    default:
      return formObject
  }
}

const modes: Mode[] = [
  {
    pointerGrid: Array.from(Array(3), () => Array.from(Array(3))),
    type: "radar",
  },
  {
    pointerGrid: Array.from(Array(3), () => Array.from(Array(1))),
    type: "htorpedo",
  },
  {
    pointerGrid: Array.from(Array(1), () => Array.from(Array(3))),
    type: "vtorpedo",
  },
  {
    pointerGrid: Array.from(Array(1), () => Array.from(Array(1))),
    type: "missile",
  },
]

function isBorder(x: number, y: number, count: number) {
  return x < 2 || x > count + 1 || y < 2 || y > count + 1
}

export function isAlreadyHit(x: number, y: number, hits: Hit[]) {
  return !!hits.filter((h) => h.x === x && h.y === y).length
}

function isSet(x: number, y: number, targetList: TargetList[], show: boolean) {
  return (
    !!targetList.filter((field) => x === field.x && y === field.y).length &&
    show
  )
}

export function targetList(
  { x: targetX, y: targetY }: Position,
  mode: number
): TargetList[] {
  const { pointerGrid, type } = modes[mode]
  const xLength = pointerGrid.length
  const yLength = pointerGrid[0].length
  return pointerGrid
    .map((arr, i) => {
      return arr.map((_, i2) => {
        const relativeX = -Math.floor(xLength / 2) + i
        const relativeY = -Math.floor(yLength / 2) + i2
        const x = targetX + (relativeX ?? 0)
        const y = targetY + (relativeY ?? 0)
        return {
          x,
          y,
          type,
          edges: [
            i === 0 ? "left" : "",
            i === xLength - 1 ? "right" : "",
            i2 === 0 ? "top" : "",
            i2 === yLength - 1 ? "bottom" : "",
          ],
        }
      })
    })
    .reduce((prev, curr) => [...prev, ...curr], [])
}

export function overlapsWithAnyBorder(target: Position, mode: number) {
  return !!targetList(target, mode).filter(({ x, y }) => isBorder(x, y, count))
    .length
}

export function composeTargetTiles(
  target: Target,
  mode: number,
  hits: Hit[]
): PointerProps[] {
  const { show } = target
  return targetList(target, mode).map((targetItem) => {
    const { x, y } = targetItem
    return {
      ...targetItem,
      show,
      imply: isAlreadyHit(x, y, hits),
    }
  })
}

export const initlialTarget = {
  preview: false,
  show: false,
  x: 2,
  y: 2,
}
export const initlialTargetPreview = {
  preview: true,
  show: false,
  x: 2,
  y: 2,
}
export const initlialMouseCursor = {
  shouldShow: false,
  x: 0,
  y: 0,
}
